package example;

import java.util.Scanner;
import java.util.Random;
import kregdb.KingsrowEgdbJNI;

import static kregdb.KingsrowEgdbJNI.*;

/**
 *
 * @author Jan-Jaap van Horssen
 * @date 2017-03-14
 * 
 * Java translation of \example\main.cpp by Ed Gilbert.
 * Added MTC test.
 * Example of how to use class KingsrowEgdbJNI.
 * 
 * See also README.txt.
 */
public class Main {

    // set these to appropriate paths on your system
    private static final String LOG_FILE      = "C:\\Maximus\\output\\kregdb.log";
    private static final String EGDB_PATH_WLD = "C:\\Program Files\\Kingsrow International\\wld_database";
    private static final String EGDB_PATH_MTC = "C:\\Program Files\\Kingsrow International\\mtc_database";

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        new Main().main();
    }

    // object variables
    
    private final Random random;
    
    private final KingsrowEgdbJNI egdb;
    
    private long wld_handle, mtc_handle;
    
    // constructor
    public Main() {
        random = new Random();
        egdb = KingsrowEgdbJNI.getInstance();
    }

    // main function, opens the databases and performs some tests
    private void main() {

        System.out.println("Initializing kingsrow egdb...");

        // open log file
        int rc = egdb.openLog(LOG_FILE);
        switch (rc) {
            case 0 : System.out.println("kregdb: log file " + LOG_FILE + " opened");
                     break;
            case 1 : System.out.println("kregdb: a log file is already open");
                     break;
            case 2 : System.out.println("kregdb: error: unable to open log file " + LOG_FILE);
                     return;
            default: System.out.println("kregdb: error: unknown return code " + rc);
                     return;
        }

        // open WLD database driver
        wld_handle = egdb.open("maxpieces=6", 1024, EGDB_PATH_WLD);
        if (wld_handle == 0L) {
            System.out.println("kregdb: error: could not open egdb driver " + EGDB_PATH_WLD);
            return;
        }        
        System.out.println("Kingsrow WLD egdb initialized (" + EGDB_PATH_WLD + ")");
                
        // open MTC database driver
        mtc_handle = egdb.open("maxpieces=6", 1024, EGDB_PATH_MTC);
        if (mtc_handle == 0L) {
            System.out.println("kregdb: error: could not open egdb driver " + EGDB_PATH_MTC);
            return;
        }
        System.out.println("Kingsrow MTC egdb initialized (" + EGDB_PATH_MTC + ")");

        // perform tests
        testAllWLD();
        test1MTC();
        testTime(10000000);     // 10 M lookups should take about 2 seconds

        // done
        System.out.print("Press enter to close");
        new Scanner(System.in).nextLine();

	egdb.close(wld_handle);
	egdb.close(mtc_handle);
        egdb.closeLog();
    }
    
    //--- test functions -------------------------------------------------------
    
    private void testAllWLD() {
	int nerrors = 0;
	System.out.printf("Testing %d positions WLD.\n", positions.length);
	for (int i = 0; i < positions.length; ++i) {
            final Position pos = positions[i].pos;
            int value = egdb.lookup(wld_handle, pos.black, pos.white, pos.king, pos.color, 0);
            if (value != positions[i].value) {
                System.out.printf("%d: lookup error, expected %d, got %d\n", i, positions[i].value, value);
                ++nerrors;
            }
	}
	System.out.printf("Test complete, %d errors.\n", nerrors);        
    }
    
    private void test1MTC() {
	int nerrors = 0;
	System.out.printf("Testing 1 position MTC.\n");
        final Position pos = positions[0].pos;
        int value = egdb.lookup(mtc_handle, pos.black, pos.white, pos.king, pos.color, 0);
        if (value != 16) {
            System.out.printf("position 0 MTC lookup error, expected %d, got %d\n", 16, value);
            ++nerrors;
        } 
	System.out.printf("Test complete, %d errors.\n", nerrors);        
    }
    
    private void testTime(int N) {
	System.out.printf("Testing %d positions WLD.\n", N);
        int nerrors = 0;
        long t0 = System.currentTimeMillis();
        for (int k=0; k<N; k++) {
            int i = random.nextInt(positions.length);
            final Position pos = positions[i].pos;
            int value = egdb.lookup(wld_handle, pos.black, pos.white, pos.king, pos.color, 0);
            if (value != positions[i].value) {
                System.out.printf("%d: lookup error, expected %d, got %d\n", i, positions[i].value, value);
                ++nerrors;
            }
        }
        long tdiff = System.currentTimeMillis() - t0;        
	System.out.printf("Test complete, %d msec %d errors, %d kN/s.\n", tdiff, nerrors, N / tdiff);        
    }
    
    //--- classes --------------------------------------------------------------

    private static class Position {

        // use Java long for Kingsrow C++ EGDB_BITBOARD
        public long black;
        public long white;
        public long king;
        public int color;  // EGDB_BLACK or EGDB_WHITE

        public Position() {
        }

        public Position(long black, long white, long king, int color) {
            this.black = black;
            this.white = white;
            this.king = king;
            this.color = color;
        }

    }

    private static class PositionValue {

        public Position pos;
        public int value;

        public PositionValue(long black, long white, long king, int color, int value) {
            this.pos = new Position(black, white, king, color);
            this.value = value;
        }

    }

    //--- test data ------------------------------------------------------------

    private static final PositionValue[] positions = {
        // added 1 position to test MTC == 16 [JJvH]
        new PositionValue(   281475245146114L,   4503599627370496L,   4785074872516608L, EGDB_BLACK, EGDB_WIN),
        // the other positions all have MTC == 1 (i.e. < threshold)
	new PositionValue(0x0000000008000000L, 0x0000000000000002L, 0x0000000008000002L, EGDB_BLACK, EGDB_DRAW),
	new PositionValue(0x0000000008000000L, 0x0000000000000002L, 0x0000000008000002L, EGDB_WHITE, EGDB_DRAW),
	new PositionValue(0x0000000008000000L, 0x0000000020000000L, 0x0000000008000000L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000000008000000L, 0x0000000020000000L, 0x0000000008000000L, EGDB_WHITE, EGDB_LOSS),
	new PositionValue(0x0000000000004000L, 0x0000000200000000L, 0x0000000000000000L, EGDB_BLACK, EGDB_DRAW),
	new PositionValue(0x0000000000004000L, 0x0000000200000000L, 0x0000000000000000L, EGDB_WHITE, EGDB_DRAW),
	new PositionValue(0x0000004000040000L, 0x0000000020000000L, 0x0000004020040000L, EGDB_BLACK, EGDB_DRAW),
	new PositionValue(0x0000004000040000L, 0x0000000020000000L, 0x0000004020040000L, EGDB_WHITE, EGDB_DRAW),
	new PositionValue(0x0000004080000000L, 0x0000000020000000L, 0x0000004080000000L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000004080000000L, 0x0000000020000000L, 0x0000004080000000L, EGDB_WHITE, EGDB_LOSS),
	new PositionValue(0x0000000009000000L, 0x0000000200000000L, 0x0000000208000000L, EGDB_WHITE, EGDB_DRAW),
	new PositionValue(0x0000000009000000L, 0x0000000800000000L, 0x0000000808000000L, EGDB_BLACK, EGDB_DRAW),
	new PositionValue(0x0000000020002000L, 0x0000000200000000L, 0x0000000020000000L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000000020002000L, 0x0000000200000000L, 0x0000000020000000L, EGDB_WHITE, EGDB_LOSS),
	new PositionValue(0x0000000080008000L, 0x0000000000000002L, 0x0000000000000002L, EGDB_BLACK, EGDB_DRAW),
	new PositionValue(0x0000000080008000L, 0x0000000000000004L, 0x0000000000000004L, EGDB_WHITE, EGDB_WIN),
	new PositionValue(0x0000004000020000L, 0x0000001000000000L, 0x0000000000000000L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000004000020000L, 0x0000001000000000L, 0x0000000000000000L, EGDB_WHITE, EGDB_LOSS),
	new PositionValue(0x0000004000040000L, 0x000000a000000000L, 0x000000e000040000L, EGDB_BLACK, EGDB_DRAW),
	new PositionValue(0x0000004000040000L, 0x000000a000000000L, 0x000000e000040000L, EGDB_WHITE, EGDB_DRAW),
	new PositionValue(0x0000004080000000L, 0x0000000020000004L, 0x0000004080000004L, EGDB_BLACK, EGDB_DRAW),
	new PositionValue(0x0000004080000000L, 0x0000000020000004L, 0x0000004080000004L, EGDB_WHITE, EGDB_DRAW),
	new PositionValue(0x0000000000000082L, 0x0000000000180000L, 0x0000000000000082L, EGDB_WHITE, EGDB_DRAW),
	new PositionValue(0x0000000000010020L, 0x0000000000180000L, 0x0000000000010020L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000000040002000L, 0x0000000200400000L, 0x0000000040400000L, EGDB_BLACK, EGDB_DRAW),
	new PositionValue(0x0000000040002000L, 0x0000000200400000L, 0x0000000040400000L, EGDB_WHITE, EGDB_DRAW),
	new PositionValue(0x0000010000000800L, 0x0001000000400000L, 0x0000010000000000L, EGDB_WHITE, EGDB_LOSS),
	new PositionValue(0x0000000200001000L, 0x0001000000400000L, 0x0000000200000000L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000800000000100L, 0x000000000a000000L, 0x0000000000000000L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000800000000100L, 0x000000000a000000L, 0x0000000000000000L, EGDB_WHITE, EGDB_LOSS),
	new PositionValue(0x0000048400000000L, 0x0000000000000004L, 0x0000048400000004L, EGDB_BLACK, EGDB_DRAW),
	new PositionValue(0x0000048400000000L, 0x0000000000000004L, 0x0000048400000004L, EGDB_WHITE, EGDB_DRAW),
	new PositionValue(0x0000100000003000L, 0x0000000020000000L, 0x0000100000003000L, EGDB_WHITE, EGDB_LOSS),
	new PositionValue(0x0000200000000088L, 0x0000000020000000L, 0x0000200000000088L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000004081000000L, 0x0000000000000004L, 0x0000004080000004L, EGDB_BLACK, EGDB_DRAW),
	new PositionValue(0x0000004081000000L, 0x0000000000000004L, 0x0000004080000004L, EGDB_WHITE, EGDB_DRAW),
	new PositionValue(0x0000010000002200L, 0x0000000200000000L, 0x0000010000000200L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000010000002200L, 0x0000000200000000L, 0x0000010000000200L, EGDB_WHITE, EGDB_LOSS),
	new PositionValue(0x0000000080008001L, 0x0008000000000000L, 0x0008000000000001L, EGDB_BLACK, EGDB_DRAW),
	new PositionValue(0x0000000080008002L, 0x0001000000000000L, 0x0001000000000002L, EGDB_WHITE, EGDB_DRAW),
	new PositionValue(0x0000004001040000L, 0x0000001000000000L, 0x0000000001000000L, EGDB_WHITE, EGDB_LOSS),
	new PositionValue(0x0000004000088000L, 0x0000001000000000L, 0x0000000000008000L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000000404100000L, 0x0000000000001000L, 0x0000000000001000L, EGDB_BLACK, EGDB_DRAW),
	new PositionValue(0x0000000404100000L, 0x0000000000001000L, 0x0000000000001000L, EGDB_WHITE, EGDB_DRAW),
	new PositionValue(0x0000028000008000L, 0x0000010000000000L, 0x0000000000000000L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000028000008000L, 0x0000010000000000L, 0x0000000000000000L, EGDB_WHITE, EGDB_LOSS),
	new PositionValue(0x0000048800000000L, 0x0000000000000003L, 0x0000048800000003L, EGDB_BLACK, EGDB_DRAW),
	new PositionValue(0x0000049000000000L, 0x0000000000140000L, 0x0000049000140000L, EGDB_WHITE, EGDB_DRAW),
	new PositionValue(0x0000100000008008L, 0x0000000020000080L, 0x0000100000008088L, EGDB_WHITE, EGDB_DRAW),
	new PositionValue(0x0000200000042000L, 0x0000000020004000L, 0x0000200000046000L, EGDB_BLACK, EGDB_DRAW),
	new PositionValue(0x0000000000000288L, 0x0000000000180000L, 0x0000000000000288L, EGDB_WHITE, EGDB_DRAW),
	new PositionValue(0x0000000010040001L, 0x0000000000180000L, 0x0000000010040001L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000004201000000L, 0x000000000a000000L, 0x000000420a000000L, EGDB_BLACK, EGDB_DRAW),
	new PositionValue(0x0000005001000000L, 0x0000000802000000L, 0x0000005802000000L, EGDB_WHITE, EGDB_DRAW),
	new PositionValue(0x0000040000002040L, 0x0000000200400000L, 0x0000040000400040L, EGDB_BLACK, EGDB_DRAW),
	new PositionValue(0x0000040000002040L, 0x0000000200400000L, 0x0000040000400040L, EGDB_WHITE, EGDB_DRAW),
	new PositionValue(0x0000100000000804L, 0x0001000000400000L, 0x0000100000000004L, EGDB_WHITE, EGDB_LOSS),
	new PositionValue(0x0000004002001000L, 0x0001000000400000L, 0x0000004002000000L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000000080008001L, 0x0000200000000100L, 0x0000200000000101L, EGDB_BLACK, EGDB_DRAW),
	new PositionValue(0x0000000080008001L, 0x0000200000000100L, 0x0000200000000101L, EGDB_WHITE, EGDB_DRAW),
	new PositionValue(0x0000004400010000L, 0x0000001800000000L, 0x0000000c00000000L, EGDB_BLACK, EGDB_DRAW),
	new PositionValue(0x0000004400010000L, 0x0000001800000000L, 0x0000000c00000000L, EGDB_WHITE, EGDB_DRAW),
	new PositionValue(0x0000610000000000L, 0x0000000005000000L, 0x0000200000000000L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000610000000000L, 0x0000000005000000L, 0x0000200000000000L, EGDB_WHITE, EGDB_LOSS),
	new PositionValue(0x0000000404100000L, 0x0000000000800020L, 0x0000000000800020L, EGDB_BLACK, EGDB_DRAW),
	new PositionValue(0x0000000404100000L, 0x0000000000800020L, 0x0000000000800020L, EGDB_WHITE, EGDB_WIN),
	new PositionValue(0x0000028000008000L, 0x0000010000000001L, 0x0000000000000001L, EGDB_BLACK, EGDB_DRAW),
	new PositionValue(0x0000028000008000L, 0x0000010000000001L, 0x0000000000000001L, EGDB_WHITE, EGDB_DRAW),
	new PositionValue(0x0000040080040000L, 0x0010000008000000L, 0x0000000000000000L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000040080040000L, 0x0010000008000000L, 0x0000000000000000L, EGDB_WHITE, EGDB_LOSS),
	new PositionValue(0x0000400024100000L, 0x0010000000000000L, 0x0010400024100000L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000400028000001L, 0x0000000040000000L, 0x0000400068000001L, EGDB_WHITE, EGDB_LOSS),
	new PositionValue(0x0000401000090000L, 0x0000000020000000L, 0x0000401000090000L, EGDB_WHITE, EGDB_LOSS),
	new PositionValue(0x0000401040000080L, 0x0000000020000000L, 0x0000401040000080L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000100001008008L, 0x0000000000000080L, 0x0000100000008088L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000100001008008L, 0x0000000000000080L, 0x0000100000008088L, EGDB_WHITE, EGDB_LOSS),
	new PositionValue(0x0000100080102000L, 0x0000000200000000L, 0x0000100080100000L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000100080102000L, 0x0000000200000000L, 0x0000100080100000L, EGDB_WHITE, EGDB_LOSS),
	new PositionValue(0x0000000080008202L, 0x0000040000000000L, 0x0000040000000202L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000000080008202L, 0x0000040000000000L, 0x0000040000000202L, EGDB_WHITE, EGDB_LOSS),
	new PositionValue(0x0000204000010004L, 0x0000001000000000L, 0x0000200000000004L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000204000010004L, 0x0000001000000000L, 0x0000200000000004L, EGDB_WHITE, EGDB_LOSS),
	new PositionValue(0x0000000404100080L, 0x0000000000000001L, 0x0000000000000081L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000000404100080L, 0x0000000000000001L, 0x0000000000000081L, EGDB_WHITE, EGDB_DRAW),
	new PositionValue(0x0000028000008001L, 0x0000010000000000L, 0x0000000000000001L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000028000008001L, 0x0000010000000000L, 0x0000000000000001L, EGDB_WHITE, EGDB_LOSS),
	new PositionValue(0x0000000444000200L, 0x0000000000000800L, 0x0000000000000800L, EGDB_BLACK, EGDB_DRAW),
	new PositionValue(0x0000000444000200L, 0x0000000000000800L, 0x0000000000000800L, EGDB_WHITE, EGDB_DRAW),
	new PositionValue(0x0000000890002000L, 0x0000400000000000L, 0x0000000000000000L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000000890002000L, 0x0000400000000000L, 0x0000000000000000L, EGDB_WHITE, EGDB_LOSS),
	new PositionValue(0x0000048400000000L, 0x0000002002000008L, 0x000004a402000008L, EGDB_BLACK, EGDB_DRAW),
	new PositionValue(0x0000048400000000L, 0x0002004020000000L, 0x000204c420000000L, EGDB_WHITE, EGDB_DRAW),
	new PositionValue(0x0000100000004020L, 0x0000000020008080L, 0x000010000000c0a0L, EGDB_WHITE, EGDB_DRAW),
	new PositionValue(0x0000100040004000L, 0x0010000420000000L, 0x0010100440004000L, EGDB_BLACK, EGDB_DRAW),
	new PositionValue(0x0000000000021008L, 0x0000000008180000L, 0x0000000008021008L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000000000021008L, 0x0000000008180000L, 0x0000000008021008L, EGDB_WHITE, EGDB_DRAW),
	new PositionValue(0x0000000050000010L, 0x0000000020028000L, 0x0000000050000010L, EGDB_WHITE, EGDB_LOSS),
	new PositionValue(0x0002000000004002L, 0x0000000008028000L, 0x0002000000004002L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000008008002000L, 0x0000400200000800L, 0x0000408008000800L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0001100000002000L, 0x0000000200100040L, 0x0001100000100040L, EGDB_WHITE, EGDB_DRAW),
	new PositionValue(0x0000100000000810L, 0x0003000000400000L, 0x0002100000000010L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000000200102000L, 0x0001000000400020L, 0x0000000200100020L, EGDB_WHITE, EGDB_DRAW),
	new PositionValue(0x0000000001040100L, 0x0004000008010000L, 0x0000000001000100L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000000001040100L, 0x0004000008010000L, 0x0000000001000100L, EGDB_WHITE, EGDB_LOSS),
	new PositionValue(0x0000400800000004L, 0x0000000005000008L, 0x000000000000000cL, EGDB_BLACK, EGDB_DRAW),
	new PositionValue(0x0000400800000004L, 0x0000000005000008L, 0x000000000000000cL, EGDB_WHITE, EGDB_DRAW),
	new PositionValue(0x0000501000000000L, 0x0000000011040000L, 0x0000001000000000L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000501000000000L, 0x0000000011040000L, 0x0000001000000000L, EGDB_WHITE, EGDB_LOSS),
	new PositionValue(0x0000c02000000000L, 0x0000001800100000L, 0x0000000000000000L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000c02000000000L, 0x0000001800100000L, 0x0000000000000000L, EGDB_WHITE, EGDB_DRAW),
	new PositionValue(0x0000400026000000L, 0x0000000000040100L, 0x0000400026040100L, EGDB_WHITE, EGDB_DRAW),
	new PositionValue(0x0000400028002000L, 0x0020000000000001L, 0x0020400028002001L, EGDB_BLACK, EGDB_DRAW),
	new PositionValue(0x0000401008000100L, 0x0000002020000000L, 0x0000403008000100L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000401008000100L, 0x0000002020000000L, 0x0000403008000100L, EGDB_WHITE, EGDB_LOSS),
	new PositionValue(0x0000000008800210L, 0x0000000000180000L, 0x0000000008800210L, EGDB_WHITE, EGDB_LOSS),
	new PositionValue(0x0008020000010001L, 0x0000000000180000L, 0x0008020000010001L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000100001004020L, 0x0000000000008080L, 0x000010000000c0a0L, EGDB_BLACK, EGDB_DRAW),
	new PositionValue(0x0000100001004020L, 0x0000000000008080L, 0x000010000000c0a0L, EGDB_WHITE, EGDB_DRAW),
	new PositionValue(0x00002000000020a0L, 0x0000020200000000L, 0x00002200000000a0L, EGDB_WHITE, EGDB_LOSS),
	new PositionValue(0x0000420400002000L, 0x0000000200000004L, 0x0000420400000004L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000004000001110L, 0x0001000000400000L, 0x0000004000000110L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000004000001110L, 0x0001000000400000L, 0x0000004000000110L, EGDB_WHITE, EGDB_LOSS),
	new PositionValue(0x0000000080008050L, 0x0002004000000000L, 0x0002004000000050L, EGDB_BLACK, EGDB_DRAW),
	new PositionValue(0x0000000080008050L, 0x0002004000000000L, 0x0002004000000050L, EGDB_WHITE, EGDB_DRAW),
	new PositionValue(0x0000204000010010L, 0x0002001000000000L, 0x0002200000000010L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000204000010010L, 0x0002001000000000L, 0x0002200000000010L, EGDB_WHITE, EGDB_LOSS),
	new PositionValue(0x0000400800000802L, 0x0000000005000000L, 0x0000000000000802L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000400800000802L, 0x0000000005000000L, 0x0000000000000802L, EGDB_WHITE, EGDB_LOSS),
	new PositionValue(0x0000000404100004L, 0x0000002040000000L, 0x0000002040000004L, EGDB_BLACK, EGDB_DRAW),
	new PositionValue(0x0000000404110000L, 0x0001002000000000L, 0x0001002000010000L, EGDB_WHITE, EGDB_DRAW),
	new PositionValue(0x0000028040002000L, 0x0000010000100000L, 0x0000000000102000L, EGDB_BLACK, EGDB_DRAW),
	new PositionValue(0x0000030000008200L, 0x0000008000000004L, 0x0000000000008004L, EGDB_WHITE, EGDB_DRAW),
	new PositionValue(0x0000044800000080L, 0x0010000008000000L, 0x0000000800000000L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000044800000080L, 0x0010000008000000L, 0x0000000800000000L, EGDB_WHITE, EGDB_LOSS),
	new PositionValue(0x0000000444000040L, 0x0000440000000000L, 0x0000440000000000L, EGDB_BLACK, EGDB_DRAW),
	new PositionValue(0x0000000448000100L, 0x0000022000000000L, 0x0000022000000000L, EGDB_WHITE, EGDB_WIN),
	new PositionValue(0x00000008c0010000L, 0x0000404000000000L, 0x0000004000000000L, EGDB_BLACK, EGDB_DRAW),
	new PositionValue(0x00000008c0010000L, 0x0000404000000000L, 0x0000004000000000L, EGDB_WHITE, EGDB_DRAW),
	new PositionValue(0x0000204200040000L, 0x0020000080000000L, 0x0000000000000000L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000204200040000L, 0x0020000080000000L, 0x0000000000000000L, EGDB_WHITE, EGDB_LOSS),
	new PositionValue(0x0000900a00008000L, 0x0000000000000080L, 0x0000900a00008080L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000900a00008000L, 0x0000000000000080L, 0x0000900a00008080L, EGDB_WHITE, EGDB_LOSS),
	new PositionValue(0x0000c00080402000L, 0x0000000020000000L, 0x0000c00080402000L, EGDB_WHITE, EGDB_LOSS),
	new PositionValue(0x0001000004108100L, 0x0000000020000000L, 0x0001000004108100L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000400801401000L, 0x0000002000000000L, 0x0000402800401000L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000400801401000L, 0x0000002000000000L, 0x0000402800401000L, EGDB_WHITE, EGDB_LOSS),
	new PositionValue(0x0000460002002000L, 0x0000000200000000L, 0x0000460002000000L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000460002002000L, 0x0000000200000000L, 0x0000460002000000L, EGDB_WHITE, EGDB_LOSS),
	new PositionValue(0x000000008000b002L, 0x0000000000000001L, 0x0000000000003003L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000000082028001L, 0x0000100000000000L, 0x0000100002020001L, EGDB_WHITE, EGDB_LOSS),
	new PositionValue(0x0000104000011004L, 0x0000001000000000L, 0x0000100000001004L, EGDB_WHITE, EGDB_LOSS),
	new PositionValue(0x000000c000020110L, 0x0000001000000000L, 0x0000008000000110L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000000404101002L, 0x0000000000002000L, 0x0000000000003002L, EGDB_WHITE, EGDB_LOSS),
	new PositionValue(0x0000000484102000L, 0x0000000000000001L, 0x0000000080002001L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000028000820100L, 0x0000010000000000L, 0x0000000000800100L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000028000820100L, 0x0000010000000000L, 0x0000000000800100L, EGDB_WHITE, EGDB_LOSS),
	new PositionValue(0x0000004444000200L, 0x0001000000000000L, 0x0001004000000000L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000002444002000L, 0x0008000000000000L, 0x0008002000000000L, EGDB_WHITE, EGDB_LOSS),
	new PositionValue(0x00000048c0010000L, 0x0000400000000000L, 0x0000004000000000L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x00000048c0010000L, 0x0000400000000000L, 0x0000004000000000L, EGDB_WHITE, EGDB_LOSS),
	new PositionValue(0x0000000085084000L, 0x0000000008000000L, 0x0000000008000000L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000000085084000L, 0x0000000008000000L, 0x0000000008000000L, EGDB_WHITE, EGDB_LOSS),
	new PositionValue(0x0000014008001100L, 0x0008000000000000L, 0x0000000000000000L, EGDB_BLACK, EGDB_WIN),
	new PositionValue(0x0000014008001100L, 0x0008000000000000L, 0x0000000000000000L, EGDB_WHITE, EGDB_LOSS),
    };

}