package kregdb;

/**
 *
 * @author Jan-Jaap van Horssen
 * @date 2017-03-14
 * 
 * Java to C++ DLL interface using Java Native Interface (JNI)
 * for Kingsrow International draughts endgame databases by Ed Gilbert
 * 
 * To use this class, you need kregdb.dll to be in the Working Directory of your
 * Java application.
 * 
 * Important Note!
 * ---------------
 * You CANNOT change this package name 'kregdb', class name 'KingsrowEgdbJNI'
 * or the native function names ('egdbOpen' etc.) or calling the DLL won't work!
 * 
 * Usage:
 * 1. call egdbOpenLog to open a log file
 * 2. call egdbOpen for each database driver you need (e.g., WLD and MTC)
 *    egdbOpen writes information to the provided log file
 * 3. call egdbLookup to lookup values in the databases
 * 4. call egdbClose for all open drivers
 * 5. call egdbCloseLog to close the log file
 *
 * See also README.txt.
 */
public class KingsrowEgdbJNI {
    
    private static final String DLL_NAME = "kregdb";   // omit extension '.dll' -- DO NOT CHANGE

    static {
        System.loadLibrary(DLL_NAME); // Load native library at runtime, kregdb.dll (Windows)
    }

    //--- Kingsrow constants ---------------------------------------------------

    // color definitions (enum EGDB_COLOR)
    public static final int EGDB_BLACK             =  0;
    public static final int EGDB_WHITE             =  1;
    
    // db values as returned by egdb_lookup() (enum WLD_VALUE)
    public static final int EGDB_SUBDB_UNAVAILABLE = -2;    // this slice is not being used
    public static final int EGDB_NOT_IN_CACHE      = -1;    // conditional lookup and position not in cache
    public static final int EGDB_UNKNOWN           =  0;    // value not in the database
    public static final int EGDB_WIN               =  1;
    public static final int EGDB_LOSS              =  2;
    public static final int EGDB_DRAW              =  3;
    public static final int EGDB_DRAW_OR_LOSS      =  4;
    public static final int EGDB_WIN_OR_DRAW       =  5;

    //--- DLL functions --------------------------------------------------------

    // Note: better not use underscores in JNI function names, this gets messy.
    private native int  egdbOpenLog (String log);
    private native long egdbOpen    (String options, int cache_mb, String dir);
    private native int  egdbLookup  (long handle, long black, long white, long king, int color, int cl);
    private native void egdbClose   (long handle);
    private native void egdbCloseLog();

    //--- class is singleton ---------------------------------------------------
    
    private static KingsrowEgdbJNI instance;
    
    public static KingsrowEgdbJNI getInstance() {
        if (instance == null) {
            instance = new KingsrowEgdbJNI();
        }
        return instance;
    }

    // constructor
    private KingsrowEgdbJNI() {
    }
    
    //--- interface to Java draughts program -----------------------------------
    
    /**
     * Open log file. There can be only one log file open at a time.
     *
     * @param logfile full path and file name of log file
     * @return return-code, 0 = log file opened successfully, 1 = log file already open, 2 = open log file failed
     */
    public int openLog(String logfile) {
        return egdbOpenLog(logfile);
    }

    /**
     * Open egdb driver for the database referred to by 'dir'.
     * A log file should be opened first by calling openLog().
     *
     * @param options Kingsrow egdb driver options, e.g. "maxpieces=6"
     * @param cache_mb cache size to be allocated in MB
     * @param dir path to the Kingsrow egdb files, such as "C:/Program Files/Kingsrow International/wld_database" or "C:/Program Files/Kingsrow International/mtc_database"
     * @return egdb driver handle 
     */
    public long open(String options, int cache_mb, String dir) {
        return egdbOpen(options, cache_mb, dir);
    }

    /**
     * Lookup a value in the database for the position (black, white, king, color).
     * 
     * @param handle egdb driver handle
     * @param black bitboard with the black pieces
     * @param white bitboard with the white pieces
     * @param king  bitboard with the king positions
     * @param color color to move, EGDB_BLACK or EGDB_WHITE
     * @param cl    conditional lookup
     * @return lookup value, one of EGDB_SUBDB_UNAVAILABLE, EGDB_NOT_IN_CACHE, EGDB_UNKNOWN, EGDB_WIN, EGDB_LOSS, EGDB_DRAW, EGDB_DRAW_OR_LOSS, EGDB_WIN_OR_DRAW
     */
    public final int lookup(long handle, long black, long white, long king, int color, int cl) {
        return egdbLookup(handle, black, white, king, color, cl);
    }
    
    /**
     * Close Kingsrow egdb driver.
     * 
     * @param handle egdb driver handle
     */
    public void close(long handle) {
        egdbClose(handle);
    }
    
    /**
     * Close log file.
     */
    public void closeLog() {
        egdbCloseLog();
    }
    
}
